// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: This class keeps track of a hockey game simulation and 
//				  its result.			  
// Version		: 1.0

import java.util.Random;

public class Game {
	// Attributes of Game class.
	String nameTeam1;
	String nameTeam2;
	int goalsTeam1;
	int goalsTeam2;
	int pointsTeam1;
	int pointsTeam2;
	
	// Initialize random number generator object.
	Random gen = new Random(System.currentTimeMillis());
	
	// Game constructor method.
	Game(Team team1, Team team2) {
		// Load team names.
		nameTeam1 = team1.getTeamName();
		nameTeam2 = team2.getTeamName();
		
		// Flag to keep track of whether the game ends in overtime or not.
		boolean overtime = false;
		
		// Compute team1's scored goals during regulation time.
		for (int i = 0; i < (team1.getTotalSkillLevel() / 50); i++)
			goalsTeam1 += team1.getRandomInRange(0, 2);
		
		if ((team1.getTotalSkillLevel() % 50) != 0)
			goalsTeam1 += team1.getRandomInRange(0, 1);
		
		// Compute team2's scored goals during regulation time.
		for (int i = 0; i < (team2.getTotalSkillLevel() / 50); i++)
			goalsTeam2 += team2.getRandomInRange(0, 2);
		
		if ((team2.getTotalSkillLevel() % 50) != 0)
			goalsTeam2 += team2.getRandomInRange(0, 1);		
		
		// If overtime is required, determine which teams scores the first goal.
		if (goalsTeam1 == goalsTeam2) {
			overtime = true;
			
			// Get the total overtime skill level for each time.
			int otTotalSkillLevelTeam1 = getOtTotalSkillLevel(team1);
			int otTotalSkillLevelTeam2 = getOtTotalSkillLevel(team2);
			
			// Determine which team scores the first goal in overtime.
			if (otTotalSkillLevelTeam1 > otTotalSkillLevelTeam2)
				goalsTeam1 += 1;
			else
				goalsTeam2 += 1;
		}
		
		// Determine the points obtained by each team. Update the stats
		// for each team.
		String losingTeamResult = "";
		if (goalsTeam1 > goalsTeam2) {
			pointsTeam1 = 2;
			if (overtime) {
				pointsTeam2 = 1;
				losingTeamResult = "OTL";
			}
			else {
				pointsTeam2 = 0;
				losingTeamResult = "L";
			}
			
			team1.updateStats(goalsTeam1, goalsTeam2, pointsTeam1, "W");
			team2.updateStats(goalsTeam2, goalsTeam1, pointsTeam2, losingTeamResult);
		}
		else {
			pointsTeam2 = 2;
			if (overtime) {
				pointsTeam1 = 1;
				losingTeamResult = "OTL";
			}
			else {
				pointsTeam1 = 0;
				losingTeamResult = "L";
			}

			team1.updateStats(goalsTeam1, goalsTeam2, pointsTeam1, losingTeamResult);
			team2.updateStats(goalsTeam2, goalsTeam1, pointsTeam2, "W");			
		}
	}

	// Ancillary methods.
	
	// Method to get the total overtime skill level for a team.
	private int getOtTotalSkillLevel(Team t) {
		int otSkillLevel = 0;
		int[] p = new int[3];    // Array of indices to 3 forwards or defensemen.
		
		// Select 3 players among forwards and defensemen.
		p = t.genUniqueRandomNums(3, 0, 20);
		
		// Add up their skill levels.
		Player[] roster = t.getRoster();
		for (int i = 0; i < 3; i++)
			otSkillLevel += roster[p[i]].getSkillLevel();
		
		// Select one of the four goalies.
		int[] g = t.genUniqueRandomNums(1, 21, 24);
		
		// Add his skill level to the total.
		otSkillLevel += roster[g[0]].getSkillLevel();
		
		// Return total overtime skill level.
		return otSkillLevel;
	}
}