// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: This program runs a simulation of an NHL hockey season
//				  in the Eastern conference.			  
// Version		: 1.0

import java.util.Scanner;
import java.util.Random;

public class NHLSimulator {
	// Initialize random number generator object.	
	private Random gen = new Random(System.currentTimeMillis());
	
	// Array of all teams in the NHL's Eastern Conference.
	private Team[] allTeams;
	
	// Array of all games to be played in an NHL season in the Eastern Conference.
	private Game[] allGames;
	
	// Method "main" drives topic selection for the NHL Simulator.
	public static void main(String[] args) {
		
		// Create a program object to allow the invocation of instance methods.
		NHLSimulator SimulatorObject = new NHLSimulator();
		
		// Display menu and wait for a menu item selection
		Scanner menuSelect = new Scanner(System.in);
		
		// Initialize menuItem selection.
		int menuItem = 0;
		
		// Flag to verify whether the season simulation has been done or not
		// (menu option 1) before selecting any of menu options 2 and 3.
		boolean simulationDone = false;
		
		while (menuItem != 9) {
			// Display menu
			System.out.println("\nNHLSimulator (Version 0.1). Author: F. de la Parra");
			System.out.println("1 - Simulate NHL Season (Eastern Conference)");
			System.out.println("2 - View Team Skill Level Profile");
			System.out.println("3 - Display End of Regular Season Table");
			System.out.println("\nSelect Option [1, 2 or 3] (9 to Quit):");
			
			// Input a menu selection (1, 2 or 3).
			boolean isInteger = false;
			Integer check = 0;
			while(!isInteger) {
				try {
					menuItem = check.parseUnsignedInt(menuSelect.next());
					isInteger = true;
				}
				catch(NumberFormatException e) {
					System.out.println("Option entered must be an integer (1, 2 or 3)!");
				}
			}
			
			// Run method corresponding to selected menu item.
			switch (menuItem) {
				
				case 1 : {
					// Invoke the Eastern Conference hockey season simulation.
					// It returns an array of "Team" with season statistics for all teams.
					SimulatorObject.runNHLSeason();
					simulationDone = true;
					break;
				}
				
				case 2 : {	
					// Invoke the displaying of the team skill levels profile method.
					if (simulationDone)
						SimulatorObject.viewSkillsProfile();
					else
						System.out.println("Must run option 1 before this option!");
					break;
				}
				
				case 3 : {
					
					// Invoke the display of the Eastern Conference end of season table.
					if (simulationDone)
						SimulatorObject.displaySeasonTable();
					else
						System.out.println("Must run option 1 before this option!");
					break;
				}
				default : {
					if (menuItem != 9)
						System.out.println("Invalid selection entered");
					break;
				}
			}
		}
		
		System.out.println("\n*** Program terminated ***\n");	
	}
	
	// This method runs a simulation of a complete Eastern Conference
	// NHL hockey season.
	
	public void runNHLSeason() {
		System.out.println("\n*** NHL Season Simulation ***\n");
		
		// Create storage for all 16 teams.
		allTeams = new Team[16];
		
		// Initialize Toronto.
		allTeams[0] = new Team();
		
		// Initialize the remaining teams.
		allTeams[1] = new Team("Boston", 5, 9, 4, 9, 5, 7);
		allTeams[2] = new Team("Buffalo", 6, 9, 4, 7, 4, 7);
		allTeams[3] = new Team("Carolina", 4, 8, 5, 7, 4, 9);
		allTeams[4] = new Team("Columbus", 4, 9, 5, 8, 7, 10);
		allTeams[5] = new Team("Detroit", 4, 7, 6, 8, 4, 6);
		allTeams[6] = new Team("Florida", 5, 7, 4, 8, 5, 9);
		allTeams[7] = new Team("Montreal", 4, 7, 4, 7, 4, 9);
		allTeams[8] = new Team("New Jersey", 4, 7, 4, 7, 5, 6);
		allTeams[9] = new Team("NY Islanders", 6, 8, 5, 7, 6, 8);
		allTeams[10] = new Team("NY Rangers", 5, 7, 4, 6, 5, 7);
		allTeams[11] = new Team("Ottawa", 4, 6, 4, 5, 4, 5);
		allTeams[12] = new Team("Philadelphia", 4, 6, 4, 6, 4, 7);
		allTeams[13] = new Team("Pittsburgh", 6, 10, 4, 7, 5, 7);
		allTeams[14] = new Team("Tampa Bay", 6, 10, 6, 10, 7, 9);
		allTeams[15] = new Team("Washington", 6, 10, 5, 8, 6, 8);
		
		// Simulate an entire hockey season in the Eastern Conference.
		allGames = new Game[240];
		int gameCount = 0;
		for (int i = 0; i < 16; i++)
			for (int j = 0; j < 16; j++)
				if (i != j) {
					allGames[gameCount] = new Game(allTeams[i], allTeams[j]);
					gameCount += 1;
				}
		
		// Find teams in first and last position after the simulation.
		int lastTeamPoints = allTeams[0].getPoints(), firstTeamPoints;
		String lastTeamName = allTeams[0].getTeamName(), firstTeamName;
		firstTeamPoints = lastTeamPoints;
		firstTeamName = lastTeamName;
		
		for (int i = 1; i < allTeams.length; i++)
			if (lastTeamPoints > allTeams[i].getPoints()) {
				lastTeamPoints = allTeams[i].getPoints();
				lastTeamName = allTeams[i].getTeamName();
			}
			else if (firstTeamPoints < allTeams[i].getPoints()) {
				firstTeamPoints = allTeams[i].getPoints();
				firstTeamName = allTeams[i].getTeamName();				
			}

		System.out.println("NHL Regular Season - Eastern Conference - 2018/2019");			
		System.out.printf("First Team: %-16sPoints:%3d\n", firstTeamName, firstTeamPoints);
		System.out.printf("Last Team : %-16sPoints:%3d\n", lastTeamName, lastTeamPoints);		
		System.out.println("\n*** NHL Season Simulation completed ***\n");
	}

	// This method displays an NHL hockey team's skill level profile. 
	
	public void viewSkillsProfile() {
		System.out.println("\n*** Viewing Team Skill Level Profile ***\n");
		
		Scanner keyboard = new Scanner(System.in);
		String teamName;
		Team currentTeam = null;
		boolean found = false;
		
		while (!found) {
			System.out.print("Enter a team name: ");
			teamName = keyboard.nextLine();
			
			if (teamName.toUpperCase().equals("Q"))
				return;
			
			for (int i = 0; i < 16; i++) {
				if (allTeams[i].getTeamName().equals(teamName)) {
					currentTeam = allTeams[i];
					found = true;
					break;
				}
			}
			
			if (found) {
				Player[] players = currentTeam.getRoster();
				System.out.println("No.  Name              Position      Skill Level");
				System.out.println("***  ************      ***********   ***********");
				for (int i = 0; i < 25; i++)
					System.out.printf("%-2d   %-15s   %-10s        %-1d\n", 
									  players[i].getNumber(), players[i].getName(), 
									  players[i].getPosition(), players[i].getSkillLevel());
			}
			else
				System.out.println("Team '" + teamName +"' not found! Please retry or press Q");
				
		}
		
		System.out.println("\n*** Viewing Team Skill Level completed ***\n");
	}
	
	// This method displays the NHL Eastern conference team statistics table
	// at the end of a hockey season.
	
	public void displaySeasonTable() {
		System.out.println("\n*** Display End of Regular Season Table ***\n");
		System.out.println("TOTAL SCORES AND STATISTICS REPORT");
		System.out.println("**********************************\n");
		System.out.println("Team Name             GP    W     L    OTL   Pts   GF    GA    Diff");
		System.out.println("******************* ***** ***** ***** ***** ***** ***** ***** ******");
		for (int i = 0; i < allTeams.length; i++)
			System.out.printf("%-18s   %3d   %3d   %3d  %3d    %3d   %3d   %3d   %3d\n",
							  allTeams[i].getTeamName(), allTeams[i].getGamesPlayed(),
							  allTeams[i].getWins(), allTeams[i].getLosses(),
							  allTeams[i].getOvertimeLosses(), allTeams[i].getPoints(),
							  allTeams[i].getGoalsInFavour(), allTeams[i].getGoalsAgainst(),
							  allTeams[i].getGoalsInFavour() - allTeams[i].getGoalsAgainst());
							
		System.out.println("\n*** End of Regular Season Table completed ***\n");
		
		// Inform the user that processing will continue after pressing 'Enter'.
		System.out.print("Press Enter to continue: ");
		
		// Wait for the user to press the 'Enter' key.
		try {
            System.in.read();
        }  
        catch(Exception e){
		} 
	}
}