// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: This class keeps track of the players and statistics
//				  of an NHL hockey team.			  
// Version		: 1.0

import java.util.Random;

public class Team {
	// Attributes of Team class.
	private String teamName;
	private Player[] roster = new Player[25];
	private int totalSkillLevel = 0;
	private int gamesPlayed = 0;
	private int wins = 0;
	private int losses = 0;
	private int overtimeLosses = 0;
	private int points = 0;
	private int goalsInFavour = 0;
	private int goalsAgainst = 0;

	// Initialize random number generator object.	
	Random gen = new Random(System.currentTimeMillis());
	
	// Constructor for the Toronto hockey team.
	Team() {
		teamName = "Toronto";
		roster[0] = new Player(28, "C. Brown", "Forward", 5);
		roster[1] = new Player(63, "T. Ennis", "Forward", 4);
		roster[2] = new Player(33, "F. Gauthier", "Forward", 5);
		roster[3] = new Player(11, "Z. Hyman", "Forward", 7);
		roster[4] = new Player(18, "A. Johnsson", "Forward", 7);
		roster[5] = new Player(43, "N. Kadri", "Forward", 7);
		roster[6] = new Player(24, "K. Kapanen", "Forward", 8);
		roster[7] = new Player(26, "P. Lindholm", "Forward", 8);
		roster[8] = new Player(12, "P. Marleau", "Forward", 8);
		roster[9] = new Player(16, "M. Marner", "Forward", 9);
		roster[10] = new Player(34, "A. Matthews", "Forward", 9);
		roster[11] = new Player(29, "W. Nylander", "Forward", 9);
		roster[12] = new Player(91, "J. Tavares", "Forward", 10);
		roster[13] = new Player(23, "T. Dermott", "Defenseman", 5);
		roster[14] = new Player(51, "J. Gardiner", "Defenseman", 4);
		roster[15] = new Player(2, "R. Hainsey", "Defenseman", 5);
		roster[16] = new Player(3, "J. Holl", "Defenseman", 6);
		roster[17] = new Player(52, "M. Marincin", "Defenseman", 4);
		roster[18] = new Player(92, "I. Ozhiganov", "Defenseman", 6);
		roster[19] = new Player(44, "M. Rielly", "Defenseman", 9);
		roster[20] = new Player(22, "N. Zaitsev", "Defenseman", 8);
		roster[21] = new Player(31, "F. Andersen", "Goalie", 10);
		roster[22] = new Player(30, "M. Hutchsinson", "Goalie", 7);
		roster[23] = new Player(50, "K. Kaskisuo", "Goalie", 5);
		roster[24] = new Player(40, "G. Sparks", "Goalie", 6);
		
		totalSkillLevel = 175;
	}
	
	// Constructor for the other teams.
	Team(String teamName, int minF, int maxF, int minD, int maxD, int minG, int maxG) {
		this.teamName = teamName;
		
		// Array to hold 25 unique random numbers between 1 and 99.
		int[] playerNumbers = genUniqueRandomNums(25, 1, 99);
		
		String playerName;    	// Variable to hold a player name.
		int skillLevel; 		// Variable to hold a player's skill level.
		
		// for (int i = 0; i < playerNumbers.length; i++)
			// System.out.println(playerNumbers[i]);
		
		// Generate the forwards.
		for (int i = 0; i < 13; i++) {
			playerName = "F" + playerNumbers[i];
			skillLevel = getRandomInRange(minF, maxF);
			roster[i] = new Player(playerNumbers[i], playerName, "Forward", skillLevel);
			totalSkillLevel	+= skillLevel;
		}
		
		//Generate the defensemen.
		for (int i = 13; i < 21; i++) {
			playerName = "D" + playerNumbers[i];
			skillLevel = getRandomInRange(minD, maxD);
			roster[i] = new Player(playerNumbers[i], playerName, "Defenseman", skillLevel);
			totalSkillLevel	+= skillLevel;		
		}
		
		// Generate the goalies.
		for (int i = 21; i < 25; i++) {
			playerName = "G" + playerNumbers[i];
			skillLevel = getRandomInRange(minG, maxG);
			roster[i] = new Player(playerNumbers[i], playerName, "Goalie", skillLevel);
			totalSkillLevel	+= skillLevel;			
		}		
	}
	
	// Accessor methods.
	
	public String getTeamName() {
		return this.teamName;
	}
	
	public Player[] getRoster() {
		return roster;
	}
	
	public int getTotalSkillLevel() {
		return this.totalSkillLevel;
	}
	
	public int getGamesPlayed() {
		return this.gamesPlayed;
	}
	
	public int getWins() {
		return this.wins;
	}

	public int getLosses() {
		return this.losses;
	}
	
	public int getOvertimeLosses() {
		return this.overtimeLosses;
	}
	
	public int getPoints() {
		return this.points;
	}
	
	public int getGoalsInFavour() {
		return this.goalsInFavour;
	}

	public int getGoalsAgainst() {
		return this.goalsAgainst;
	}

	// Ancillary methods.
	
	// Method to get "maxCount" unique random integer in range [minInt, maxInt].
	public int[] genUniqueRandomNums(int maxCount, int minInt, int maxInt) {
		// Compute the range in which the unique random integers will be generated.
		int range = maxInt - minInt + 1;
		
		// The range's size must be greater than the max. count of random integers to generate.
		if (maxCount > range) {
			System.out.println("Cannot generate " + maxCount + "unique random " +
							   "integers in interval [" + minInt + "," + maxInt + "] !");
			System.out.println("Program aborted");
			System.exit(0);
		}
		
		// Array to return the "maxCount" unique random integers
		int[] uniqueNums = new int[maxCount];
		
		int nextRandomNumber;		// Stores next random integer
		int count =0;
		int i = 0;
		
		// First random integer generated cannot be a duplicate.
		uniqueNums[0] = minInt + gen.nextInt(range);
		
		// Generate the remaining (maxCount - 1) random integers.
		while (count < maxCount) {
			nextRandomNumber = minInt + gen.nextInt(range);
			for (i = 0; i < count; i++)
				if (uniqueNums[i] == nextRandomNumber)
					break;

			if (i == count && uniqueNums[i] != nextRandomNumber) {
				count += 1;
				if (count < maxCount) {
				uniqueNums[count] = nextRandomNumber;
				}
			}	
		}
		
		return uniqueNums;
	}
	
	// Method to get a random integer in the range [minInt, maxInt].
	public int getRandomInRange(int minInt, int maxInt) {
		// Compute the range in which the unique random integers will be generated.
		int range = maxInt - minInt + 1;
		// Return random integer in specified range.
		return  minInt + gen.nextInt(range);
	}
	
	// Method to update the season statistics for the two teams involved
	// in the current game.	
	public void updateStats(int goalsInFavour, int goalsAgainst, int points, String result) {
		this.goalsInFavour += goalsInFavour;
		this.goalsAgainst += goalsAgainst;
		this.points += points;
		this.gamesPlayed += 1;
		
		if (result.equals("W"))
			wins += 1;
		else if (result.equals("L"))
			losses += 1;
		else if (result.equals("OTL"))
			overtimeLosses += 1;
	}
}